from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='Nameservers',
    icon='globe',
    dependencies=[
        PluginDependency('main')
    ],
)


def init():
    import resolv



from ajenti.api import *
from ajenti.util import cache_value


@plugin
class ServiceMultiplexor (object):
    """
    Merges together output of all available ServiceManagers.
    """
    def init(self):
        self.managers = ServiceManager.get_all()

    @cache_value(1)
    def get_all(self):
        """
        Returns all :class:`Service` s.
        """
        r = []
        for mgr in self.managers:
            r += mgr.get_all()
        return r

    def get_one(self, name):
        """
        Returns a :class:`Service` by name.
        """
        for mgr in self.managers:
            s = mgr.get_one(name)
            if s:
                return s
        return None


@interface
class ServiceManager (object):
    def get_all(self):
        return []

    def get_one(self, name):
        """
        Returns a :class:`Service` by name.
        """
        for s in self.get_all():
            if s.name == name:
                return s
        return None


class Service (object):
    source = 'unknown'
    """ Marks which ServiceManager owns this object """

    def __init__(self):
        self.name = None
        self.running = False

    @property
    def icon(self):
        return 'play' if self.running else None

    def start(self):
        pass

    def stop(self):
        pass

    def restart(self):
        pass

    def command(self, cmd):
        pass



import subprocess

from ajenti.api import *

from api import *
from nctp_linux import *


@plugin
class DebianNetworkConfig (LinuxIfconfig, INetworkConfig):
    platforms = ['debian']

    interfaces = None

    def __init__(self):
        self.rescan()

    def rescan(self):
        self.interfaces = {}

        try:
            f = open('/etc/network/interfaces')
            ss = f.read().splitlines()
            f.close()
        except IOError, e:
            return

        auto = []

        while len(ss) > 0:
            line = ss[0].strip(' \t\n')
            if (len(line) > 0 and not line[0] == '#'):
                a = line.split(' ')
                for s in a:
                    if s == '':
                        a.remove(s)
                if (a[0] == 'auto'):
                    auto.append(a[1])
                elif (a[0] == 'allow-hotplug'):
                    pass
                elif (a[0] == 'iface'):
                    tmp = NetworkInterface()
                    tmp.addressing = a[3]
                    tmp.type = a[2]
                    e = self.get_iface(a[1], self.detect_iface_bits(tmp))
                    del tmp
                    e.type = a[2]
                    e.addressing = a[3]
                    e.devclass = self.detect_dev_class(e)
                    try:
                        e.up = 'UP' in subprocess.check_output(['ifconfig', e.name])
                    except:
                        e.up = False
                else:
                    e.params[a[0]] = ' '.join(a[1:])
            if len(ss) > 1:
                ss = ss[1:]
            else:
                ss = []

        for x in auto:
            if x in self.interfaces:
                self.interfaces[x].auto = True

    def get_iface(self, name, bits):
        if not name in self.interfaces:
            self.interfaces[name] = NetworkInterface()
            self.interfaces[name].bit_classes = bits
            self.interfaces[name].name = name
        return self.interfaces[name]

    def save(self):
        f = open('/etc/network/interfaces', 'w')
        for i in self.interfaces:
            self.save_iface(self.interfaces[i], f)
        f.close()
        return

    def save_iface(self, iface, f):
        if iface.auto:
            f.write('auto ' + iface.name + '\n')
        f.write('iface %s %s %s\n' % (iface.name, iface.type, iface.addressing))
        for x in iface.params:
            f.write('\t%s %s\n' % (x, iface.params[x]))
        f.write('\n')



import ajenti
from ajenti.api import *
from ajenti.plugins.webserver_common.api import WebserverPlugin


@plugin
class Apache (WebserverPlugin):
    service_name = 'apache2'
    service_buttons = [
        {
            'command': 'force-reload',
            'text': _('Reload'),
            'icon': 'step-forward',
        }
    ]
    hosts_available_dir = '/etc/apache2/sites-available'
    hosts_enabled_dir = '/etc/apache2/sites-enabled'

    template = """<VirtualHost *:80>
    ServerAdmin webmaster@localhost

    DocumentRoot /var/www

    <Directory />
            Options FollowSymLinks
            AllowOverride None
    </Directory>

    <Directory /var/www/>
            Options Indexes FollowSymLinks MultiViews
            AllowOverride None
            Order allow,deny
            allow from all
    </Directory>
</VirtualHost>
"""

    def init(self):
        self.title = 'Apache'
        self.category = _('Software')
        self.icon = 'globe'
        if ajenti.platform == 'centos':
            self.service_name = 'httpd'



from __future__ import unicode_literals

import atexit
import gettext
import locale
import logging
import os
import signal
import socket
import subprocess
import syslog
import sys
import time
import tempfile

import ajenti
from ajenti.http import HttpRoot
from ajenti.routing import CentralDispatcher
from ajenti.middleware import SessionMiddleware, AuthenticationMiddleware
import ajenti.console
import ajenti.plugins
import ajenti.feedback

import gevent
from gevent import monkey
monkey.patch_all(select=False)
from socketio.server import SocketIOServer


class SSLTunnel (object):
    """
    gevent's SSL server implementation is buggy so we have to use stunnel
    """

    def start(self, host, port, certificate_path):
        self.port = self.get_free_port()
        logging.info('Starting SSL tunnel for port %i' % self.port)
        stunnel = 'stunnel'
        if subprocess.call(['which', 'stunnel4']) == 0:
            stunnel = 'stunnel4'
        cfg = tempfile.NamedTemporaryFile(delete=False)
        cfg.write("""
            cert = %s
            foreground = yes
            pid =

            [default]
            accept = %s:%i
            connect = 127.0.0.1:%i
        """ % (
            certificate_path,
            host or '0.0.0.0', port,
            self.port
        ))
        cfg.close()
        cmd = [
            stunnel,
            cfg.name,
        ]
        self.process = subprocess.Popen(cmd, stdout=None)
        self._filename = cfg.name

    def check(self):
        time.sleep(0.5)
        return self.process.poll() is None

    def stop(self):
        os.unlink(self._filename)
        self.process.terminate()
        if self.check():
            self.process.kill()

    def get_free_port(self):
        s = socket.socket()
        s.bind(('', 0))
        port = s.getsockname()[1]
        s.close()
        return int(port)


def run():
    try:
        locale.setlocale(locale.LC_ALL, '')
    except:
        logging.warning('Couldn\'t set default locale')
    localedir = os.path.abspath(os.path.join(os.path.split(ajenti.core.__file__)[0], 'locale'))
    gettext.textdomain('ajenti')
    gettext.install('ajenti', localedir, unicode=True)

    logging.info('Ajenti %s running on platform: %s' % (ajenti.version, ajenti.platform))

    if ajenti.debug:
        ajenti.console.register()

    # Load plugins
    ajenti.plugins.manager.load_all()

    bind_spec = (ajenti.config.tree.http_binding.host, ajenti.config.tree.http_binding.port)
    if ':' in bind_spec[0]:
        addrs = socket.getaddrinfo(bind_spec[0], bind_spec[1], socket.AF_INET6, 0, socket.SOL_TCP)
        bind_spec = addrs[0][-1]

    ssl_tunnel = None
    if ajenti.config.tree.ssl.enable:
        ssl_tunnel = SSLTunnel()
        ssl_tunnel.start(bind_spec[0], bind_spec[1], ajenti.config.tree.ssl.certificate_path)
        if ssl_tunnel.check():
            logging.info('SSL tunnel running fine')
            bind_spec = ('127.0.0.1', ssl_tunnel.port)
            atexit.register(ssl_tunnel.stop)
        else:
            logging.error('SSL tunnel failed to start')

    # Fix stupid socketio bug (it tries to do *args[0][0])
    socket.socket.__getitem__ = lambda x, y: None

    logging.info('Starting server on %s' % (bind_spec, ))
    listener = socket.socket(socket.AF_INET6 if ':' in bind_spec[0] else socket.AF_INET, socket.SOCK_STREAM)
    listener.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    listener.bind(bind_spec)
    listener.listen(10)

    stack = [SessionMiddleware(), AuthenticationMiddleware(), CentralDispatcher()]
    ajenti.server = SocketIOServer(
        listener,
        log=open(os.devnull, 'w'),
        application=HttpRoot(stack).dispatch,
        policy_server=False,
        resource='ajenti:socket',
    )

    # auth.log
    try:
        syslog.openlog(
            ident=str(b'ajenti'),
            facility=syslog.LOG_AUTH,
        )
    except:
        syslog.openlog(b'ajenti')

    try:
        gevent.signal(signal.SIGTERM, lambda: sys.exit(0))
    except:
        pass

    ajenti.feedback.start()
    ajenti.server.serve_forever()

    if hasattr(ajenti.server, 'restart_marker'):
        logging.warn('Restarting by request')
        if ssl_tunnel:
            ssl_tunnel.stop()

        fd = 20  # Close all descriptors. Creepy thing
        while fd > 2:
            try:
                os.close(fd)
                logging.debug('Closed descriptor #%i' % fd)
            except:
                pass
            fd -= 1

        os.execv(sys.argv[0], sys.argv)
    else:
        logging.info('Stopped by request')



import psutil

from ajenti.api import plugin
from ajenti.api.sensors import Sensor
from ajenti.plugins.dashboard.api import DashboardWidget
from ajenti.util import str_fsize


@plugin
class MemorySensor (Sensor):
    id = 'memory'
    timeout = 5

    def measure(self, variant):
        v = psutil.virtual_memory()
        return (v.total - v.available, v.total)


@plugin
class SwapSensor (Sensor):
    id = 'swap'
    timeout = 5

    def measure(self, variant):
        v = psutil.swap_memory()
        return (v.used, v.total)


@plugin
class MemoryWidget (DashboardWidget):
    name = _('Memory usage')
    icon = 'tasks'

    def init(self):
        self.sensor = Sensor.find('memory')
        self.append(self.ui.inflate('sensors:progressbar-widget'))
        self.find('icon').icon = 'tasks'
        self.find('name').text = _('Memory usage')
        value = self.sensor.value()
        self.find('value').text = str_fsize(value[0])
        self.find('progress').value = 1.0 * value[0] / value[1]


@plugin
class SwapWidget (DashboardWidget):
    name = _('Swap usage')
    icon = 'hdd'

    def init(self):
        self.sensor = Sensor.find('swap')
        self.append(self.ui.inflate('sensors:progressbar-widget'))
        self.find('icon').icon = 'hdd'
        self.find('name').text = _('Swap usage')
        value = self.sensor.value()
        self.find('value').text = str_fsize(value[0])
        if value[1] > 0:
            frac = 1.0 * value[0] / value[1]
        else:
            frac = 0
        self.find('progress').value = frac



import os
import gevent
import requests
import json
from datetime import datetime

from ajenti.api import *
from ajenti.api.sensors import Sensor
from ajenti.plugins import manager
from ajenti.plugins.configurator.api import ClassConfigEditor
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
import ajenti


ENDPOINT = 'http://ajenti.local.org:8080/docking-bay/receive/%i'
#ENDPOINT = 'http://ajenti.org/docking-bay/receive/%i'
#

def _check():
    return os.path.exists('/var/lib/ajenti/.ajenti.org.enabled')


@plugin
class AjentiOrgReporterConfigEditor (ClassConfigEditor):
    title = 'ajenti.org reporter'
    icon = 'globe'

    def init(self):
        self.append(self.ui.inflate('ajenti_org:config'))


@plugin
class AjentiOrgReporter (BasePlugin):
    default_classconfig = {'key': None}
    classconfig_editor = AjentiOrgReporterConfigEditor
    classconfig_root = True

    @classmethod
    def verify(cls):
        return _check()

    def init(self):
        self.last_report = None
        self.last_error = None
        gevent.spawn(self.worker)

    @classmethod
    def classinit(cls):
        cls.get()

    def get_key(self):
        self.load_classconfig()
        if not 'key' in self.classconfig:
            return None
        return self.classconfig['key']

    def set_key(self, key):
        self.classconfig['key'] = key
        self.save_classconfig()

    def worker(self):
        if not ENDPOINT:
            return
        while True:
            datapack = {'sensors': {}}

            for sensor in Sensor.get_all():
                data = {}
                for variant in sensor.get_variants():
                    data[variant] = sensor.value(variant)
                datapack['sensors'][sensor.id] = data

            gevent.sleep(10)
            url = ENDPOINT % ajenti.config.tree.installation_id

            if not self.get_key():
                continue

            try:
                requests.post(url, data={
                    'data': json.dumps(datapack),
                    'key': self.get_key()
                })
                self.last_report = datetime.now()
                self.last_error = None
            except Exception, e:
                self.last_error = e


@plugin
class AjentiOrgSection (SectionPlugin):

    @classmethod
    def verify(cls):
        return _check()

    def init(self):
        self.title = 'Ajenti.org'
        self.icon = 'group'
        self.category = ''
        self.order = 25
        self.append(self.ui.inflate('ajenti_org:main'))
        self.reporter = AjentiOrgReporter.get(manager.context)

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        key = self.reporter.get_key()
        self.find('pane-not-configured').visible = key is None
        self.find('pane-configured').visible = key is not None
        self.find('last-report').text = str(self.reporter.last_report)
        self.find('last-error').visible = self.reporter.last_error is not None
        self.find('last-error').text = str(self.reporter.last_error)

    @on('save-key', 'click')
    def on_save_key(self):
        AjentiOrgReporter.get().set_key(self.find('machine-key').value)
        self.refresh()

    @on('disconnect', 'click')
    def on_disconnect(self):
        AjentiOrgReporter.get().set_key(None)
        self.refresh()



from ajenti.api import *
from ajenti.api.http import HttpPlugin, url
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on, UIElement, p
from ajenti.ui.binder import Binder

import requests
import urllib2

from client import MuninClient
from widget import MuninWidget


@plugin
class Munin (SectionPlugin):
    def init(self):
        self.title = 'Munin'
        self.icon = 'stethoscope'
        self.category = _('Software')
        self.append(self.ui.inflate('munin:main'))

        def post_graph_bind(o, c, i, u):
            for plot in u.nearest(lambda x: x.typeid == 'munin:plot'):
                plot.on('widget', self.on_add_widget, i)

        self.find('graphs').post_item_bind = post_graph_bind

        self.munin_client = MuninClient.get()
        self.binder = Binder(None, self)

    def on_page_load(self):
        self.refresh()

    def on_add_widget(self, graph, url=None, period=None):
        self.context.launch('dashboard-add-widget', cls=MuninWidget, config={'url': url, 'period': period})

    def refresh(self):
        self.munin_client.reset()
        try:
            self.munin_client.fetch_domains()
        except requests.ConnectionError, e:
            self.find_type('tabs').active = 1
            self.context.notify('error', _('Couldn\'t connect to Munin: %s') % e.message)
        except Exception, e:
            self.find_type('tabs').active = 1
            if e.message == 'auth':
                self.context.notify('error', _('Munin HTTP authentication failed'))
            else:
                raise
        self.binder.reset(self.munin_client).autodiscover().populate()

    @on('save-button', 'click')
    def save(self):
        self.binder.update()
        self.munin_client.save_classconfig()
        self.refresh()
        self.find_type('tabs').active = 0


@plugin
class MuninProxy (HttpPlugin):
    def init(self):
        self.client = MuninClient.get()

    @url('/ajenti:munin-proxy/(?P<url>.+)')
    def get_page(self, context, url):
        if context.session.identity is None:
            context.respond_redirect('/')
        url = urllib2.unquote(url)
        data = requests.get(self.client.classconfig['prefix'] + url,
                auth=(self.client.classconfig['username'], self.client.classconfig['password'])
               ).content
        context.add_header('Content-Type', 'image/png')
        context.respond_ok()
        return data


@p('url', bindtypes=[str, unicode])
@p('period')
@p('widget', type=bool)
@plugin
class MuninPlot (UIElement):
    typeid = 'munin:plot'



import sys
import signal
import logging
import traceback
from ajenti.util import public

@public
def register():
    logging.info('Debug mode: press Ctrl-\\ to enter interactive console')
    signal.signal(signal.SIGQUIT, handle_quit)
    sys.excepthook = handle_exception


def handle_exception(type, value, tb):
    print 'FATAL CRASH'
    print value
    print '\n'.join(traceback.format_tb(tb))
    launch()


def handle_quit(*args):
    print '\nSIGQUIT received at:'
    traceback.print_stack()
    launch()


@public
def launch(*args):
    print '\nActivating emergency console\n'

    import readline
    import code
    import ajenti
    from ajenti.plugins import manager

    vars = globals().copy()
    vars.update(locals())
    shell = code.InteractiveConsole(vars)

    print 'Press Ctrl-D to exit console'
    print 'Plugin manager is available as \'manager\'\n'
    shell.interact()



import os

from ajenti.api import *
from ajenti.plugins import *
from ajenti.ui import *


@p('buttons', default=['ok'], type=eval)
@plugin
class Dialog (UIElement):
    typeid = 'dialog'

    def init(self):
        presets = {
            'ok': {'text': 'OK', 'id': 'ok'},
            'cancel': {'text': 'Cancel', 'id': 'cancel'},
        }
        new_buttons = []
        for button in self.buttons:
            if type(button) == str and button in presets:
                new_buttons.append(presets[button])
            else:
                new_buttons.append(button)
        self.buttons = new_buttons


@p('text', default='Input value:', type=unicode)
@p('value', default='', bindtypes=[str, unicode], type=unicode)
@plugin
class InputDialog (UIElement):
    typeid = 'inputdialog'

    def on_button(self, button):
        if button == 'ok':
            self.reverse_event('submit', {'value': self.value})
        else:
            self.reverse_event('cancel', {})
        self.visible = False


class BaseFileDialog (object):
    def navigate(self, path):
        self.path = path
        self.verify_path()
        self.refresh()

    def verify_path(self):
        if not self.path.startswith(self.root):
            self.path = self.root

    def refresh(self):
        pass


@p('_files', default=[], type=list)
@p('_dirs', default=[], type=list)
@p('path', default='/', type=unicode)
@p('root', default='/', type=unicode)
@plugin
class OpenFileDialog (UIElement, BaseFileDialog):
    typeid = 'openfiledialog'

    def init(self):
        self.verify_path()
        self.refresh()

    def on_item_click(self, item):
        path = os.path.abspath(os.path.join(self.path, item))
        if os.path.isdir(path):
            self.navigate(path)
        else:
            self.reverse_event('select', {'path': path})

    def on_button(self, button=None):
        self.reverse_event('select', {'path': None})

    def refresh(self):
        self._dirs = []
        self._files = []
        if len(self.path) > 1:
            self._dirs.append('..')
        for item in sorted(os.listdir(self.path)):
            if os.path.isdir(os.path.join(self.path, item)):
                self._dirs.append(item)
            else:
                self._files.append(item)


@p('_dirs', default=[], type=list)
@p('path', default='/', type=unicode)
@p('root', default='/', type=unicode)
@plugin
class OpenDirDialog (UIElement, BaseFileDialog):
    typeid = 'opendirdialog'

    def init(self):
        self.verify_path()
        self.on('item-click', self.on_item_click)
        self.refresh()

    def on_item_click(self, item):
        path = os.path.abspath(os.path.join(self.path, item))
        if os.path.exists(path):
            self.navigate(path)

    def on_button(self, button=None):
        if button == 'select':
            self.reverse_event('select', {'path': self.path})
        else:
            self.reverse_event('select', {'path': None})

    def refresh(self):
        self._dirs = []
        if len(self.path) > 1:
            self._dirs.append('..')
        for item in sorted(os.listdir(self.path)):
            if os.path.isdir(os.path.join(self.path, item)):
                self._dirs.append(item)


@p('_dirs', default=[], type=list)
@p('path', default='/', type=unicode)
@p('root', default='/', type=unicode)
@plugin
class SaveFileDialog (UIElement, BaseFileDialog):
    typeid = 'savefiledialog'

    def init(self):
        self.on('item-click', self.on_item_click)
        self.refresh()

    def on_item_click(self, item):
        path = os.path.abspath(os.path.join(self.path, item))
        if os.path.isdir(path):
            self.navigate(path)
        else:
            self.reverse_event('select', {'path': path})

    def navigate(self, path):
        self.path = path
        self.verify_path()
        self.refresh()

    def on_button(self, button=None):
        self.reverse_event('select', {'path': None})

    def refresh(self):
        self._dirs = []
        self._files = []
        if len(self.path) > 1:
            self._dirs.append('..')
        for item in sorted(os.listdir(self.path)):
            if os.path.isdir(os.path.join(self.path, item)):
                self._dirs.append(item)



import ajenti
from ajenti.api import *
from passlib.hash import sha512_crypt


def restrict(permission):
    """
    Marks a decorated function as requiring ``permission``. If the invoking user doesn't have one, :class:`SecurityError` is raised.
    """
    def decorator(fx):
        def wrapper(*args, **kwargs):
            UserManager.get().require_permission(permission)
            return fx(*args, **kwargs)
        return wrapper
    return decorator


class SecurityError (Exception):
    """
    Indicates that user didn't have a required permission.

    .. attribute:: permission

        permission ID
    """

    def __init__(self, permission):
        self.permission = permission

    def __str__(self):
        return 'Permission "%s" required' % self.permission


@plugin
class UserManager (object):
    def check_password(self, username, password):
        """
        Verifies the given username/password combo
        """

        if not username in ajenti.config.tree.users:
            return False
        type = 'plain'
        saved = ajenti.config.tree.users[username].password
        if '|' in saved:
            type, saved = saved.split('|')

        if type == 'plain':
            hash = password
            return hash == saved
        elif sha512_crypt.identify(saved):
            return sha512_crypt.verify(password, saved)

    def hash_password(self, password):
        if not password.startswith('sha512|'):
            password = 'sha512|%s' % sha512_crypt.encrypt(password)
        return password

    def require_permission(self, permission):
        """
        Checks current user for given permissing and raises :class:`SecurityError` if he doesn't have one
        """

        context = extract_context()
        if context.user.name == 'root':
            return
        if not permission in context.user.permissions:
            raise SecurityError(permission)


@interface
class PermissionProvider (object):
    """
    Override to create your own set of permissions
    """

    def get_permissions(self):
        """ Should return a list of permission names """
        return []

    def get_name(self):
        """ Should return a human-friendly name for this set of permissions (displayed in Configurator) """
        return ''


__all__ = ['restrict', 'PermissionProvider', 'SecurityError', 'UserManager']



#!/usr/bin/env python

from distutils.core import setup
from setuptools import find_packages

import ajenti.build

__version = ajenti.build.version

exclusion = [
    'ajenti.plugins.elements',
    'ajenti.plugins.ltfs',
    'ajenti.plugins.test*',
]

setup(
    name='ajenti',
    version=__version,
    install_requires=[
        'pyOpenSSL',
        'gevent',
        'gevent-socketio',
        'lxml>=2.2.4',
        'python-daemon',
        'passlib',
        'psutil>=0.6.0',
        'python-catcher',
        'requests',
        'reconfigure',
        'PIL',
    ],
    description='The server administration panel',
    author='Eugeny Pankov',
    author_email='e@ajenti.org',
    url='http://ajenti.org/',
    packages=find_packages(exclude=['reconfigure', 'reconfigure.*'] + exclusion),
    package_data={'': ['content/*.*', 'content/*/*.*', 'content/*/*/*.*', 'layout/*.*', 'locale/*/*/*.mo']},
    scripts=['ajenti-panel', 'ajenti-ssl-gen'],
    data_files=[
        ('/etc/ajenti', ['packaging/files/config.json']),
        ('/etc/init.d', ['packaging/files/ajenti']),
        ('/var/lib/ajenti/plugins', ['packaging/files/.placeholder']),
    ],
)
